//
//  AppDelegate.m
//  crafttest
//
//  Created by Bilahari on 10/5/17.
//  Copyright © 2017 logitech. All rights reserved.
//

#import "AppDelegate.h"
#import "Craft.h"
#import "BACraftEventHandler.h"


@interface AppDelegate ()

@property (weak) IBOutlet NSWindow *window;
@property LogiCraftClient *client;
@property (weak) IBOutlet NSSlider *horizontalSlider;
@property (weak) IBOutlet NSSlider *verticalSlider;
@property (weak) IBOutlet NSSlider *circularSlider;
@end


@implementation AppDelegate

-(void) handleCraftEventViaAppDelegate:(NSDictionary *)msg_dict {
    
    NSSlider *selectedSlider = nil;
    NSString *selectedSliderType = nil;
    
    if ([msg_dict[@"message_type"] isEqualToString:@"crown_turn_event"]) {
        selectedSliderType =  msg_dict[@"task_options"][@"current_tool_option"];
        if ([selectedSliderType isEqualToString:@"horizontal"])
            selectedSlider = self.horizontalSlider;
        else if ([selectedSliderType isEqualToString:@"vertical"])
            selectedSlider = self.verticalSlider;
        else if ([selectedSliderType isEqualToString:@"circular"])
            selectedSlider = self.circularSlider;
        else
            selectedSlider = self.circularSlider;
        
        double value = selectedSlider.doubleValue;
        value += [msg_dict[@"delta"] doubleValue];
        if (value < 0) value = 0;
        [selectedSlider setDoubleValue:value];
        
        // send the updated value back so that you can see it on on-screen overlays (real time user feedback)
        [self.client sendMessage:@
         {
             @"message_type":@"tool_update",
             @"payload" : @ {
                 @"tool": @"slider",
                 @"tool_option": selectedSliderType,
                 @"tool_option_value" : @(value),
                 @"play_task" : @"" // look at SDK documentation for more info on individual fields
             }
         }];
    }
}

-(void) appendToTextViewLog:(NSString *)message {
    dispatch_async(dispatch_get_main_queue(), ^{
        NSAttributedString* attr = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"=> %@\n", message]];
        
        NSTextView *tv = self.textArea;
        NSTextStorage *ts = tv.textStorage;
        [ts appendAttributedString:attr];
        [tv scrollRangeToVisible:NSMakeRange([[tv string] length], 0)];
    });
}

- (IBAction)onUnregisterButtonClick:(id)sender {
    [self.client disconnect];
}

- (IBAction)onRegisterButtonClick:(id)sender {
    // if already connected, disconnect and start new connection
    if (self.client) {
        [self.client disconnect];
        self.client = nil;
    }

    self.client = [[LogiCraftClient alloc] initWithConnectionType:LOGI_WS];
    self.client.delegate = [[BACraftEventHandler alloc] init];
    [self.client connectWithUUID:@"36f0bda7-7562-4c58-8a25-ddb00e9ebabd"];
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
    
    [self.horizontalSlider setMinValue:0];
    [self.horizontalSlider setMaxValue:150];
    [self.verticalSlider setMinValue:0];
    [self.verticalSlider setMaxValue:500];
    [self.circularSlider setMinValue:0];
    [self.circularSlider setMaxValue:250];
}

- (void)applicationWillTerminate:(NSNotification *)aNotification {
    // Insert code here to tear down your application
}

@end
